package com.unipi.talepis.androidprotalspring1;

import org.springframework.web.bind.annotation.*;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
public class DBControlller {
    private static String url = "jdbc:sqlite:src/main/resources/Databases/Greece2021.db";
    @RequestMapping("/destinations")
    public String selectAllDestinations(){
        String sql = "SELECT Title FROM Destination";
        StringBuilder builder = new StringBuilder();
        try {
            Connection connection = DriverManager.getConnection(url);
            Statement statement = connection.createStatement();
            ResultSet resultSet = statement.executeQuery(sql);
            while (resultSet.next()){
                builder.append(resultSet.getString("Title")).append("<br/>");
            }
            statement.close();
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }
        return builder.toString();
    }
    @RequestMapping("/getDestinations")
    public List<Destination> getDestinations(){
        String sql = "SELECT * FROM Destination";
        List<Destination> destinations = new ArrayList<>();
        try {
            Connection connection = DriverManager.getConnection(url);
            Statement statement = connection.createStatement();
            ResultSet resultSet = statement.executeQuery(sql);
            while (resultSet.next()){
                Destination destination = new Destination();
                destination.setId(resultSet.getInt("Id"));
                destination.setTitle(resultSet.getString("Title"));
                destination.setDescription(resultSet.getString("Description"));
                destination.setLocation(resultSet.getString("Location"));
                destination.setCost(resultSet.getInt("Cost"));
                destinations.add(destination);
            }
            statement.close();
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }
        return destinations;
    }
    @GetMapping("Destination/{id}")
    public Destination getDestination(@PathVariable Integer id){
        String sql = "SELECT * FROM Destination WHERE Id=?";
        Destination destination = null;
        try {
            Connection connection = DriverManager.getConnection(url);
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setInt(1,id);
            ResultSet resultSet = preparedStatement.executeQuery();
            if (resultSet.next()){
                destination = new Destination();
                destination.setId(resultSet.getInt("Id"));
                destination.setTitle(resultSet.getString("Title"));
                destination.setDescription(resultSet.getString("Description"));
                destination.setLocation(resultSet.getString("Location"));
                destination.setCost(resultSet.getInt("Cost"));
            }
            preparedStatement.close();
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }
        return destination;
    }
    @DeleteMapping("Destination/{id}")
    public Map<String,String> deleteDestination(@PathVariable Integer id){
        String sql = "DELETE FROM Destination WHERE Id=?";
        String message = null;
        try {
            Connection connection = DriverManager.getConnection(url);
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setInt(1,id);
           int deletedDestinations = preparedStatement.executeUpdate();
            if (deletedDestinations>0){
                message = "Deleted "+String.valueOf(deletedDestinations)+" destination";
            }else {
                message = "The id you provided was not valid";
            }
            preparedStatement.close();
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }
        Map<String,String> jsonMessage = new HashMap<>();
        jsonMessage.put("message",message);
        return jsonMessage;
    }
    @PostMapping("newDestination")
    public Map<String,String> newDestination(@RequestBody Map<String,String> body){
        int id = Integer.parseInt(body.get("id"));
        String title = body.get("title");
        String description = body.get("description");
        String location = body.get("location");
        int cost = Integer.parseInt(body.get("cost"));

        String sql = "INSERT INTO Destination VALUES(?,?,?,?,?)";
        String message = null;
        try {
            Connection connection = DriverManager.getConnection(url);
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setInt(1,id);
            preparedStatement.setString(2,title);
            preparedStatement.setString(3,description);
            preparedStatement.setString(4, location);
            preparedStatement.setInt(5,cost);
            int insertedDestinations = preparedStatement.executeUpdate();
            if (insertedDestinations>0){
                message = "Inserted "+String.valueOf(insertedDestinations)+" destination";
            }
            preparedStatement.close();
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
            message = throwables.getLocalizedMessage();
        }
        Map<String,String> jsonMessage = new HashMap<>();
        jsonMessage.put("message",message);
        return jsonMessage;
    }
    @PutMapping("updateDestination/{id}")
    public Map<String,String> updateDestination(@PathVariable int id,@RequestBody Map<String,String> body){

        String title = body.get("title");
        String description = body.get("description");
        String location = body.get("location");
        int cost = Integer.parseInt(body.get("cost"));

        String sql = "UPDATE Destination " +
                "SET Title=?," +
                "Description=?," +
                "Location=?," +
                "Cost=? " +
                "WHERE Id=?";
        String message = null;
        try {
            Connection connection = DriverManager.getConnection(url);
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1,title);
            preparedStatement.setString(2,description);
            preparedStatement.setString(3, location);
            preparedStatement.setInt(4,cost);
            preparedStatement.setInt(5,id);
            int updatedDestinations = preparedStatement.executeUpdate();
            if (updatedDestinations>0){
                message = "Updated "+String.valueOf(updatedDestinations)+" destination";
            }
            preparedStatement.close();
            connection.close();
        } catch (SQLException throwables) {
            throwables.printStackTrace();
            message = throwables.getLocalizedMessage();
        }
        Map<String,String> jsonMessage = new HashMap<>();
        jsonMessage.put("message",message);
        return jsonMessage;
    }
}
